@@ -5,7 +5,7 @@
    http://developers.exosite.com/display/OP/Remote+Procedure+Call+API
 
 Usage:
-  exo [options] <command> [<args> ...]
+  exo [--help] [options] <command> [<args> ...]
 
 Commands:
   read
@@ -20,8 +20,10 @@ Commands:
   listing
   info
   flush
+  usage
   tree
   script
+  spark
 
 Options:
   --host=<host>        OneP URL. Default is $EXO_HOST or m2.exosite.com
@@ -31,7 +33,7 @@ Options:
   -h --help            Show this screen
   -v --version         Show version
 
-See 'exo <command> -h' for more information on a specific command.
+See 'exo <command> --help' for more information on a specific command.
 """
 
 # Copyright (c) 2013, Exosite, LLC
@@ -42,6 +44,7 @@ import json
 import csv
 import re
 from datetime import datetime
+from datetime import timedelta
 import time
 from pprint import pprint
 from collections import OrderedDict
@@ -49,6 +52,7 @@ import itertools
 import math
 
 from docopt import docopt
+from dateutil import parser
 from onepv1lib import onep
 from onepv1lib import onep_exceptions
 try:
@@ -67,8 +71,7 @@ Options:
     --limit=<limit>          number of data points to read [default: 1]
     --selection=all|autowindow|givenwindow  downsample method [default: all]
     --format=csv|raw         output format [default: csv]
-    --timeformat=unix|human  unix timestamp or human-readable? [default: human]
-    --intervals              show distribution of intervals between points''',
+    --timeformat=unix|human  unix timestamp or human-readable? [default: human]''',
     'write':
         '''Write data at the current time.\n\nUsage:
     exo [options] write <cik> [<rid>] --value=<value>''',
@@ -143,6 +146,16 @@ Options:
     'flush':
         '''Remove all time series data from a resource.\n\nUsage:
     exo [options] flush <cik> [<rid>]''',
+    'usage':
+        '''Display usage of One Platform resources over a time period.\n\nUsage:
+    exo [options] usage <cik> [<rid>] --start=<time> [--end=<time>]
+
+    <time> can be a unix timestamp or formatted like any of these:
+
+    2011-10-23T08:00:00-07:00 10/1/2012 "2012-10-23 14:01"
+
+    If time part is omitted, it assumes 00:00:00.
+    To report through the present time, pass --end=now or omit --end entirely.''',
     'tree':
         '''Display a resource's descendants.\n\nUsage:
     exo tree [--verbose] [--hide-keys] <cik>''',
@@ -150,7 +163,12 @@ Options:
     exo [options] script <script-file> <cik> ...
 
 Options:
-    --name=<name>  script name, if different from script filename.'''
+    --name=<name>  script name, if different from script filename.''',
+    'spark': '''Show distribution of intervals between points.\n\nUsage:
+    exo [options] spark <cik> [<rid>] --days=<days>
+
+Options:
+    --stddev=<num>  exclude intervals more than num standard deviations from mean'''
 }
 
 for k in cmd_doc:
@@ -191,12 +209,28 @@ class ExoRPC():
             # TODO: does this always indicate an error condition?
             raise RPCException(', '.join(['{}: {}'.format(msg, t) for msg, t in response]))
 
-    def read(self, cik, rid, limit, sort='asc', starttime=None, selection='all'):
+    def _raise_for_deferred(self, responses):
+        r = []
+        for call, isok, response in responses:
+            self._raise_for_response(isok, response)
+            r.append(response)
+        return r
+
+    def read(self,
+             cik,
+             rid,
+             limit,
+             sort='asc',
+             starttime=None,
+             endtime=None,
+             selection='all'):
         options = {'limit': limit,
                'sort': sort,
                'selection': selection}
         if starttime is not None:
-            options['starttime'] = starttime
+            options['starttime'] = int(starttime)
+        if endtime is not None:
+            options['endtime'] = int(endtime)
         isok, response = self.exo.read(
             cik,
             rid,
@@ -252,9 +286,7 @@ class ExoRPC():
             self.exo.drop(cik, rid, defer=True)
 
         if self.exo.has_deferred(cik):
-            responses = self.exo.send_deferred(cik)
-            for call, isok, response in responses:
-                self._raise_for_response(isok, response)
+            self._raise_for_deferred(self.exo.send_deferred(cik))
 
     def map(self, cik, rid, alias):
         isok, response = self.exo.map(cik, rid, alias)
@@ -285,10 +317,9 @@ class ExoRPC():
             for rid in type_list:
                 self.exo.info(cik, rid, defer=True)
 
+        responses = []
         if self.exo.has_deferred(cik):
-            responses = self.exo.send_deferred(cik)
-            for call, isok, response in responses:
-                self._raise_for_response(isok, response)
+            responses = self._raise_for_deferred(self.exo.send_deferred(cik))
 
         # From the return values make a list of dicts like this:
         # [{'<rid0>':<info0>, '<rid1>':<info1>}, {'<rid2>':<info2>}, [], {'<rid3>': <info3>}]
@@ -317,11 +348,20 @@ class ExoRPC():
     def flush(self, cik, rids):
         for rid in rids:
             self.exo.flush(cik, rid, defer=True)
-
         if self.exo.has_deferred(cik):
-            responses = self.exo.send_deferred(cik)
-            for call, isok, response in responses:
-                self._raise_for_response(isok, response)
+            self._raise_for_deferred(self.exo.send_deferred(cik))
+
+    def usage(self, cik, rid, metrics, start, end):
+        for metric in metrics:
+            self.exo.usage(cik, rid, metric, start, end, defer=True)
+        responses = []
+        if self.exo.has_deferred(cik):
+            responses = self._raise_for_deferred(self.exo.send_deferred(cik))
+        # show report
+        maxlen = max([len(m) for m in metrics])
+        for i, r in enumerate(responses):
+            print("{}:{} {}".format(
+                  metrics[i], ' ' * (maxlen - len(metrics[i])), r))
 
     def _disp_key(self, cli_args, k):
         if cli_args['--hide-keys']:
@@ -548,10 +588,12 @@ def format_time(sec):
                     [60, 'm']]
     text = ""
     for s, label in intervals:
-        if sec > s:
+        if sec >= s and sec / s > 0:
             text = "{} {}{}".format(text, sec / s, label)
             sec -= s * (sec / s)
-    return "{} {}s".format(text, sec).strip()
+    if sec > 0:
+        text += " {}s".format(sec)
+    return text.strip()
 
 
 def spark(numbers, empty_val=None):
@@ -598,19 +640,40 @@ def spark(numbers, empty_val=None):
 
     return ''.join(out)
 
-def show_intervals(er, cik, rid, limit):
+def meanstdv(l):
+    '''Calculate mean and standard deviation'''
+    n, mean, std = len(l), 0, 0
+    mean = sum(l) / float(len(l))
+    std = math.sqrt(sum([(x - mean)**2 for x in l]) / (len(l) - 1))
+    return mean, std
+
+
+def show_intervals(er, cik, rid, start, end, limit, numstd=None):
     # show a distribution of intervals between data
     data = er.read(cik,
                    rid,
+                   limit,
                    sort='desc',
-                   limit=limit)
+                   starttime=start,
+                   endtime=end)
 
+    if len(data) == 0:
+        return
     intervals = [data[i - 1][0] - data[i][0] for i in xrange(1, len(data))]
     intervals = sorted(intervals)
+
+    if numstd is not None:
+        # only include data within numstd standard deviations
+        # of the mean
+        mean, std = meanstdv(intervals)
+        intervals = [x for x in intervals
+                    if mean - numstd * std <= x
+                    and x <= mean + numstd * std]
+        if len(intervals) == 0:
+            return
     num_bins = 60
     min_t, max_t = min(intervals), max(intervals)
     bin_size = float(max_t - min_t) / num_bins * 1.0
-
     bins = []
     for i in range(num_bins):
         bin_min = min_t + i * bin_size
@@ -646,12 +709,6 @@ def read_cmd(er, cik, rids, args):
     limit = args['--limit']
     limit = 1 if limit is None else int(limit)
 
-    # --intervals is practically another command. Only
-    # shares time specifications (see above).
-    if args['--intervals']:
-        show_intervals(er, cik, rid, limit)
-        return
-
     timeformat = args['--timeformat']
     dw = csv.DictWriter(sys.stdout, ['timestamp', 'value'])
     fmt = args['--format']
@@ -846,12 +903,42 @@ def handle_args(cmd, args):
             pr(json.dumps(info))
     elif cmd == 'flush':
         er.flush(cik, rids)
+    elif cmd == 'usage':
+        allmetrics = ['client',
+                      'dataport',
+                      'datarule',
+                      'dispatch',
+                      'email',
+                      'http',
+                      'sms',
+                      'xmpp']
+        start = args['--start']
+        end = args['--end']
+        parse_ts = lambda(s): int(time.mktime(parser.parse(s).timetuple()))
+        is_ts = lambda(s): re.match('^[0-9]+$', s) is not None
+        if is_ts(start):
+            start = int(start)
+        else:
+            start = parse_ts(start)
+        if end is None or end == 'now':
+            end = int(time.mktime(datetime.now().timetuple()))
+        elif is_ts(end):
+            end = int(end)
+        else:
+            end = parse_ts(end)
+        er.usage(cik, rids[0], allmetrics, start, end)
     # special commands
-    elif cmd == 'tree':
-        er.tree(cik, cli_args=args)
+    elif cmd == 'tree': er.tree(cik, cli_args=args)
     elif cmd == 'script':
         # cik is a list of ciks
         er.upload(cik, args['<script-file>'], args['--name'])
+    elif cmd == 'spark':
+        days = int(args['--days'])
+        end = time.mktime(datetime.now().timetuple())
+        start = time.mktime((datetime.now() - timedelta(days=days)).timetuple())
+        numstd = args['--stddev']
+        numstd = int(numstd) if numstd is not None else None
+        show_intervals(er, cik, rids[0], start, end, limit=1000000, numstd=numstd)
     else:
         raise ExoException("Command not handled")
 
@@ -873,7 +960,7 @@ def cmd(argv=None, stdin=None, stdout=None, stderr=None):
 
     args = docopt(
         __doc__,
-        version="Exosite RPC API Command Line {}".format(__version__),
+        version="Exosite Command Line {}".format(__version__),
         options_first=True)
 
     # get command args
